@@ -25,25 +25,22 @@ namespace SparkleLib {
     // listens for change notifications
     public abstract class SparkleListenerBase {
 
-        public event ConnectedEventHandler Connected;
-        public delegate void ConnectedEventHandler ();
+        public event Action Connected = delegate { };
+        public event Action Disconnected = delegate { };
 
-        public event DisconnectedEventHandler Disconnected;
-        public delegate void DisconnectedEventHandler ();
-
-        public event ReceivedEventHandler Received;
-        public delegate void ReceivedEventHandler (SparkleAnnouncement announcement);
+        public event AnnouncementReceivedEventHandler AnnouncementReceived = delegate { };
+        public delegate void AnnouncementReceivedEventHandler (SparkleAnnouncement announcement);
 
         public readonly Uri Server;
 
-
         public abstract void Connect ();
         public abstract bool IsConnected { get; }
         public abstract bool IsConnecting { get; }
+
+
         protected abstract void AnnounceInternal (SparkleAnnouncement announcent);
         protected abstract void AlsoListenToInternal (string folder_identifier);
 
-
         protected List<string> channels = new List<string> ();
 
 
@@ -52,11 +49,7 @@ namespace SparkleLib {
         private Dictionary<string, List<SparkleAnnouncement>> recent_announcements =
             new Dictionary<string, List<SparkleAnnouncement>> ();
 
-        private Dictionary<string, SparkleAnnouncement> queue_up   =
-            new Dictionary<string, SparkleAnnouncement> ();
-
-        private Dictionary<string, SparkleAnnouncement> queue_down =
-            new Dictionary<string, SparkleAnnouncement> ();
+        private Dictionary<string, SparkleAnnouncement> queue_up   = new Dictionary<string, SparkleAnnouncement> ();
 
         private Timer reconnect_timer = new Timer {
             Interval = 60 * 1000,
@@ -82,36 +75,31 @@ namespace SparkleLib {
         {
             if (!IsRecentAnnouncement (announcement)) {
                 if (IsConnected) {
-                    SparkleHelpers.DebugInfo ("Listener",
-                        "Announcing message " + announcement.Message + " to " +
-                        announcement.FolderIdentifier + " on " + Server);
+                    SparkleLogger.LogInfo ("Listener", "Announcing message " + announcement.Message +
+                        " to " + announcement.FolderIdentifier + " on " + Server);
 
                     AnnounceInternal (announcement);
                     AddRecentAnnouncement (announcement);
 
                 } else {
-                    SparkleHelpers.DebugInfo ("Listener",
-                        "Can't send message to " +
-                        Server + ". Queuing message");
-
+                    SparkleLogger.LogInfo ("Listener", "Can't send message to " + Server + ". Queuing message");
                     this.queue_up [announcement.FolderIdentifier] = announcement;
                 }
 
             } else {
-                SparkleHelpers.DebugInfo ("Listener",
-                    "Already processed message " + announcement.Message + " to " +
-                    announcement.FolderIdentifier + " from " + Server);
+                SparkleLogger.LogInfo ("Listener", "Already processed message " + announcement.Message +
+                    " to " + announcement.FolderIdentifier + " from " + Server);
             }
         }
 
 
         public void AlsoListenTo (string channel)
         {
-            if (!this.channels.Contains (channel) && IsConnected) {
-                SparkleHelpers.DebugInfo ("Listener",
-                    "Subscribing to channel " + channel + " on " + Server);
-
+            if (!this.channels.Contains (channel))
                 this.channels.Add (channel);
+
+            if (IsConnected) {
+                SparkleLogger.LogInfo ("Listener", "Subscribing to channel " + channel + " on " + Server);
                 AlsoListenToInternal (channel);
             }
         }
@@ -119,64 +107,49 @@ namespace SparkleLib {
 
         public void Reconnect ()
         {
-            SparkleHelpers.DebugInfo ("Listener", "Trying to reconnect to " + Server);
+            SparkleLogger.LogInfo ("Listener", "Trying to reconnect to " + Server);
             Connect ();
         }
 
 
         public void OnConnected ()
         {
-            SparkleHelpers.DebugInfo ("Listener", "Listening for announcements on " + Server);
+            foreach (string channel in this.channels.GetRange (0, this.channels.Count)) {
+                SparkleLogger.LogInfo ("Listener", "Subscribing to channel " + channel + " on " + Server);
+                AlsoListenToInternal (channel);
+            }
 
-            if (Connected != null)
-                Connected ();
+            SparkleLogger.LogInfo ("Listener", "Listening for announcements on " + Server);
+            Connected ();
 
             if (this.queue_up.Count > 0) {
-                SparkleHelpers.DebugInfo ("Listener",
-                    "Delivering " + this.queue_up.Count + " queued messages...");
+                SparkleLogger.LogInfo ("Listener", "Delivering " + this.queue_up.Count + " queued messages...");
 
                 foreach (KeyValuePair<string, SparkleAnnouncement> item in this.queue_up) {
                     SparkleAnnouncement announcement = item.Value;
                     Announce (announcement);
                 }
-
-                this.queue_down.Clear ();
             }
         }
 
 
         public void OnDisconnected (string message)
         {
-            SparkleHelpers.DebugInfo ("Listener", "Disconnected from " + Server + ": " + message);
-
-            if (Disconnected != null)
-                Disconnected ();
+            SparkleLogger.LogInfo ("Listener", "Disconnected from " + Server + ": " + message);
+            Disconnected ();
         }
 
 
         public void OnAnnouncement (SparkleAnnouncement announcement)
         {
-            SparkleHelpers.DebugInfo ("Listener",
-                "Got message " + announcement.Message + " from " +
+            SparkleLogger.LogInfo ("Listener", "Got message " + announcement.Message + " from " +
                 announcement.FolderIdentifier + " on " + Server);
 
-            if (IsRecentAnnouncement (announcement)) {
-                SparkleHelpers.DebugInfo ("Listener",
-                    "Ignoring previously processed message " + announcement.Message + 
-                    " from " + announcement.FolderIdentifier + " on " + Server);
-                
-                  return;
-            }
-
-            SparkleHelpers.DebugInfo ("Listener",
-                "Processing message " + announcement.Message + " from " +
-                announcement.FolderIdentifier + " on " + Server);
+            if (IsRecentAnnouncement (announcement))
+                return;
 
             AddRecentAnnouncement (announcement);
-            this.queue_down [announcement.FolderIdentifier] = announcement;
-
-            if (Received != null)
-                Received (announcement);
+            AnnouncementReceived (announcement);
         }
 
 
@@ -188,15 +161,11 @@ namespace SparkleLib {
 
         private bool IsRecentAnnouncement (SparkleAnnouncement announcement)
         {
-            if (!this.recent_announcements
-                    .ContainsKey (announcement.FolderIdentifier)) {
-
+            if (!this.recent_announcements.ContainsKey (announcement.FolderIdentifier)) {
                 return false;
 
             } else {
-                foreach (SparkleAnnouncement recent_announcement in
-                         GetRecentAnnouncements (announcement.FolderIdentifier)) {
-
+                foreach (SparkleAnnouncement recent_announcement in GetRecentAnnouncements (announcement.FolderIdentifier)) {
                     if (recent_announcement.Message.Equals (announcement.Message))
                         return true;
                 }
@@ -211,7 +180,7 @@ namespace SparkleLib {
             if (!this.recent_announcements.ContainsKey (folder_identifier))
                 this.recent_announcements [folder_identifier] = new List<SparkleAnnouncement> ();
 
-            return (List<SparkleAnnouncement>) this.recent_announcements [folder_identifier];
+            return this.recent_announcements [folder_identifier];
         }
 
 
@@ -224,8 +193,7 @@ namespace SparkleLib {
                 recent_announcements.Add (announcement);
 
             if (recent_announcements.Count > this.max_recent_announcements)
-                recent_announcements.RemoveRange (0,
-                    (recent_announcements.Count - this.max_recent_announcements));
+                recent_announcements.RemoveRange (0, recent_announcements.Count - this.max_recent_announcements);
         }
     }
 }
